#!/usr/bin/env python
#
#  Copyright 2010, 2011 G24
#
#    This file is part of gpy.
#
#    gpy is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    gpy is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with gpy.  If not, see <http://www.gnu.org/licenses/>.

import os
import math
import time
import socket
import ConfigParser
from datetime import datetime 

class core:
    def dispatch(self,str):
        """Do not use!"""
        self.irc.send(str + '\n\r')
        self.ok("/" + str)
    def send(self,cmd,target,msg):
        self.dispatch(cmd + ' ' + target + ' :' + msg)
    def say(self,target,msg):
        self.send(self.config.get("connection","msgtype"),target,msg)
    def reply(self,msg):
        if self.recvinfo["target"] == self.vars["nick"]:
            self.say(self.recvinfo["origin.nick"],msg)
        else:
            self.say(self.recvinfo["target"],self.recvinfo["origin.nick"] + ": " + msg)
    def saypm(self,msg):
        self.send("PRIVMSG",target,msg)
    def cancel(self,line):
        print("\033[91m{0}\033[0m".format(line))
        self.log('! ' + line)
    def ok(self,line):
        print("\033[92m{0}\033[0m".format(line))
        self.log('  ' + line)
    def info(self,line):
        print("\033[93m{0}\033[0m".format(line))
        self.log('? ' + str(line))
    def log(self,line):
        log = open(self.logpath,"a")
        log.write("{0} {1}\n".format(str(datetime.now()),line))
class connection(core):
    def __init__(self):
        # Constant.
        self.version = __import__('gpy').__version__
        print("Starting gpy {0}...".format(self.version))
        self.directory = os.environ['HOME'] + '/.gpy/'
        self.configpath = self.directory + 'main.cfg'
        self.logpath = self.directory + 'main.log'
        if not os.path.exists(self.directory):
            os.makedirs(self.directory)
        self.commands = []
        self.vars = {}

        if os.path.isfile(self.configpath) == False:
            self.makeconf()
        
        self.config = ConfigParser.RawConfigParser()
        self.config.read(self.configpath)

        self.mainclasses = {};
        self.modules = {};
        tmp = (self.config.get("me","modules").strip() + ",").split(",")
        for each in tmp[0:(len(tmp) - 1)]:
            try:
                self.modules = __import__("gpy", globals(), locals(), [each], -1)
                self.mainclasses[each] = getattr(getattr(self.modules,each),"mod_main")(self)
                self.ok("Module {0} loaded.".format(each))
            except  Exception, e:
                self.cancel("Module {0} not loaded. {1}".format(each,e))
                pass
        # Connecting.
        self.info("Connecting to {0} ... ".format(self.config.get("connection","network")))
        self.irc=socket.socket()
        try:
            self.irc.connect ((self.config.get("connection","network"), self.config.getint("connection","port")))
            self.dispatch('NICK ' + self.config.get("connection","nick"))
            self.dispatch("USER " + self.config.get("connection","nick") + " * * :" + self.version+" http://repo.or.cz/w/gpy.git ; Command char: "+str(self.config.get("me","char")))
        except Exception, e:
            self.cancel("Connection failed. {0}".format(e))
            return
        if self.config.get("connection","ns_name") != "":
            self.dispatch("PRIVMSG NICKSERV :id {0} {1}".format(self.config.get("connection","ns_name"),self.config.get("connection","ns_pwd")))
        
        self.vars["nick"] = self.config.get("connection","nick")

        while True:
            self.data = self.irc.recv(4096)
            while(self.data[len(self.data)-2:len(self.data)]) != '\r\n':
                self.data += self.irc.recv(4096)
            for each in self.data.split("\r\n"):
                if each == "": # Do nothing with empty lines
                    break
                if each.split(' ')[0] == 'PING': # PING
                    self.dispatch('PONG ' + each.split()[1])
                    break
                self.parseline(each)
                if len(each.strip()) != 0: # Now listen.
                    self.listen()
                    try:
                        for mod in self.mainclasses:
                            self.mainclasses[mod].listen()
                    except:
                        pass
    def makeconf(self):
        self.config = ConfigParser.RawConfigParser()
        self.config.read(self.configpath)
        self.config.add_section("connection")
        self.config.set("connection",'network',raw_input("IRC network to connect to: "))
        self.config.set("connection",'channels',raw_input("Channels to join (comma-separated, no spaces): "))
        self.config.set("connection",'nick',raw_input("My nickname: "))
        self.config.set("connection",'msgtype',raw_input("Message type (privmsg or notice): "))
        self.config.set("connection",'ns_name',raw_input("NickServ username (if there is none, press ENTER): "))
        self.config.set("connection",'ns_pwd',raw_input("NickServ password (if there is none, press ENTER): "))
        self.config.set("connection",'port',raw_input("Port (if unsure, type 6667): "))
        self.config.add_section("users")
        username=raw_input("Your username (used for bot administration, I will ask for password too): ")
        self.config.set("users",username,raw_input("Your password: "))
        self.config.add_section("admins")
        self.config.set("admins",username,'')
        self.config.add_section("me")
        self.config.set("me","char",raw_input("My command char: "))
        self.config.set("me","modules",raw_input("Modules to load (comma-separated, no spaces, must include 'admin' module): "))
        while self.config.get("me","modules").find('admin') == -1:
            self.config.set("me","modules",raw_input("Modules to load must include 'admin' module, try again: "))
        self.config.add_section("google")
        self.config.set("google","cx",raw_input("Google cx (leave blank if you didn't load web module; see http://code.google.com/apis/customsearch/v1/using_rest.html ): "))
        self.config.set("google","key",raw_input("Google key (details and usage same as cx): "))
        with open(self.configpath, 'wb') as configfile:
            self.config.write(configfile)
    def listen(self):
        # If we've read the MOTD - JOIN channels.
        if self.recvinfo["cmd"] == "MODE":
            self.info(self.recvinfo)
            for each in (self.config.get("connection","channels").strip() + ",").split(","):
                if each != "":
                    self.dispatch('JOIN ' + each)
            return
        if self.recvinfo["message"] == self.config.get("me","char"):
            self.reply("Modules loaded: {0}".format(str(self.mainclasses.keys())))
            return
        if self.recvinfo["message.command"] in self.mainclasses and self.recvinfo["message.params"] == "":
            mod=self.mainclasses[self.recvinfo["message.command"]]
            self.reply("{0} Includes commands: {1}.".format(mod.__doc__,str(mod.cmds.keys()) ))
            return
        if self.recvinfo["cmd"] == "INVITE":
            self.dispatch("JOIN "+self.recvinfo["message"])
            return
        if self.recvinfo["cmd"] == "NICK" and self.recvinfo["origin.nick"] == self.vars["nick"]:
            self.vars["nick"] = self.recvinfo["target"].replace(":","")
            return
    def writeconfig(self):
        with open(self.configpath, 'w') as configfile:
            self.config.write(configfile)
    def parseline(self,line):
        self.recvinfo = {}
        self.recvinfo["origin"] = line.split(" ")[0].split(":")[1]
        self.recvinfo["origin.nick"] = ""
        self.recvinfo["origin.ident"] = ""
        self.recvinfo["origin.host"] = ""
        if self.recvinfo["origin"].find("@") != -1:
            self.recvinfo["origin.nick"] = self.recvinfo["origin"].split("!")[0]
            self.recvinfo["origin.ident"] = self.recvinfo["origin"].split("!")[1].split("@")[0]
            self.recvinfo["origin.host"] = self.recvinfo["origin"].split("!")[1].split("@")[1]
        self.recvinfo["cmd"] = line.split(" ")[1]
        self.recvinfo["target"] = line.split(" ")[2]
        self.recvinfo["message"] = ":".join(" ".join(line.split(" ")[3:]).split(":")[1:]).strip() #-echo Hello world.
        self.recvinfo["message.command"] = ""
        self.recvinfo["message.params"] = ""
        if self.recvinfo["message"] != "":
            if self.recvinfo["message"][0] == self.config.get("me","char"):
                tmp = self.recvinfo["message"].replace(self.config.get("me","char"),"",1) # echo Hello world.
                self.recvinfo["message.command"] = tmp.split(" ")[0] # echo
                self.recvinfo["message.params"] = " ".join(tmp.split(" ")[1:]).strip() # Hello world.
class clisten():
    def __init__(self,c):
        self.conn=c
        if hasattr(self,"addcmds"):
            self.addcmds()
    def listen(self):
        if (self.conn.recvinfo["message.command"] in self.cmds) == False:
            return
        if self.conn.recvinfo["message.params"] == "":
            self.conn.reply(getattr(self,self.cmds[self.conn.recvinfo["message.command"]]).__doc__)
        else:
            try:
                getattr(self,self.cmds[self.conn.recvinfo["message.command"]])()
            except Exception,e:
                self.conn.reply(str(e))
        if hasattr(self,"clisten"):
            self.clisten()
